# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility functions that modify a response.

These are most commonly used to inject code in declarative command
implementations.
"""


def flatten_aggregated_list(field_name: str):
  """Collect resources returned from an aggregated list response into a list.

  The aggregated list returns every instance of the resource in every scope
  in a particular form. A partial example is given below:
    {
      "kind": "compute#storagePoolAggregatedList",
      "id": "projects/cengizk-staging/aggregated/storagePools",
      "items": {
        "global": {
          "warning": {
            "code": "NO_RESULTS_ON_PAGE",
            "message": "There are no results for scope 'global' on this page.",
            "data": [
              {
                "key": "scope",
                "value": "global"
              }
            ]
          }
        },
        "zones/us-central1-ir1": {
          "storagePools": [
            {
              "kind": "compute#storagePool",
              "id": "2014221622075011414",
              "creationTimestamp": "2023-03-23T10:43:21.767-07:00",
              "name": "foo",
            },
            {
              "kind": "compute#storagePool",
              "id": "5926595245683775672",
              "creationTimestamp": "2023-04-05T09:48:24.926-07:00",
              "name": "foo2",
              "description": "",
              "sizeGb": "1",
              "provisionedIops": "5",
              "zone": "https://www.googleapis.com/compute/staging_alpha/
                projects/cengizk-staging/zones/us-central1-ir1",
              "state": "READY",
              "type": "SSD",
              "resourceStatus": {
                "numberOfDisks": "0",
                "aggregateDiskSizeGb": "0",
                "maxAggregateDiskSizeGb": "5",
                "usedBytes": "0",
                "usedReducedBytes": "0",
                "aggregateDiskProvisionedIops": "0",
                "lastResizeTimestamp": "1969-12-31T16:00:00.000-08:00"
              },
              {
                "kind": "compute#storagePool",
                "id": "8565305401462476119",
                "creationTimestamp": "2025-06-11T23:29:12.208-07:00",
                "name": "dennisx-exapool",
                "description": "test description",
                "zone":
                "https://www.googleapis.com/compute/staging_alpha/projects/dennisx-staging/zones/us-central2-oj1",
                "state": "READY",
                "selfLink":
                "https://www.googleapis.com/compute/staging_alpha/projects/dennisx-staging/zones/us-central2-oj1/storagePools/dennisx-exapool-2",
                "selfLinkWithId":
                "https://www.googleapis.com/compute/staging_alpha/projects/dennisx-staging/zones/us-central2-oj1/storagePools/8565305401462476119",
                "storagePoolType":
                "https://www.googleapis.com/compute/staging_alpha/projects/dennisx-staging/zones/us-central2-oj1/storagePoolTypes/exapool-hyperdisk-balanced",
                "status": {
                  "diskCount": "2",
                  "poolUsedCapacityBytes": "1258019648",
                  "poolUserWrittenBytes": "21474836480",
                  "totalProvisionedDiskCapacityGb": "82400",
                  "maxTotalProvisionedDiskCapacityGb": "102400",
                  "totalProvisionedDiskIops": "20000",
                  "totalProvisionedDiskThroughput": "2000",
                  "exapoolMaxReadIops": "35840",
                  "exapoolMaxWriteIops": "5120",
                  "exapoolMaxReadThroughput": "665",
                  "exapoolMaxWriteThroughput": "51"
                },
                "capacityProvisioningType": "ADVANCED",
                "performanceProvisioningType": "ADVANCED",
                "exapoolProvisionedCapacityGb": {
                  "writeOptimized": "0",
                  "readOptimized": "0",
                  "capacityOptimized": "10240"
                }
              }
            }

  Args:
    field_name:  Name of the field that the resource is collected into. For the
      example above, for the zone "us-central1-ir1", the resources are collected
      into "storagePools", thus that must be the value. In general, this value
      should be the plural camelcase form of the resource type.

  Returns:
    A function that can serve as a response modification hook. See
    `modify_response_hooks` and `python_hook` in `yaml_command_schema.yaml`.
  """
  def hook(response, _):
    resources = []

    for scope in response.items.additionalProperties:
      resources += getattr(scope.value, field_name)

    return resources

  return hook



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing network policies."""

from typing import ClassVar

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.network_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.network_policies import flags


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Describe(base.DescribeCommand):
  """Describe a Compute Engine network policy.

  *{command}* is used to describe network policies. A network
  policy is a set of rules that classifies network traffic.
  """

  NETWORK_POLICY_ARG: ClassVar[compute_flags.ResourceArgument]

  @classmethod
  def Args(cls, parser):
    cls.NETWORK_POLICY_ARG = flags.NetworkPolicyArgument(
        required=True, operation='describe'
    )
    cls.NETWORK_POLICY_ARG.AddArgument(parser, operation_type='describe')

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.NETWORK_POLICY_ARG.ResolveAsResource(args, holder.resources)

    network_policy = client.NetworkPolicy(ref, compute_client=holder.client)

    return network_policy.Describe(only_generate_request=False)


Describe.detailed_help = {
    'EXAMPLES': (
        """\
    To describe a network policy with name ``my-policy'',
    in region ``my-region'', run:

      $ {command} my-policy --region=my-region
    """
    ),
}



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list environments in a project and location."""


from googlecloudsdk.api_lib.composer import environments_util as environments_api_util
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.composer import flags
from googlecloudsdk.command_lib.composer import resource_args
from googlecloudsdk.core import resources


DETAILED_HELP = {
    'EXAMPLES':
        """\
          To list the Cloud Composer environments under the project 'project-1'
          and in location 'us-central1', run:

            $ {command} --project=project-1 --locations=us-central1
        """
}


@base.UniverseCompatible
class List(base.ListCommand):
  """List the Cloud Composer environments under a project and location.

  List environments that have not been successfully deleted. Prints a table
  with the following columns:
  * name
  * location
  * status
  * creation timestamp
  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def _GetUri(environment):
    r = resources.REGISTRY.ParseRelativeName(
        environment.name,
        collection='composer.projects.locations.environments',
        api_version='v1',
    )
    return r.SelfLink()

  @staticmethod
  def Args(parser):
    resource_args.AddLocationResourceArg(
        parser,
        'in which to list environments',
        positional=False,
        required=arg_parsers.ArgRequiredInUniverse(
            default_universe=False, non_default_universe=True
        ),
        plural=True,
        help_supplement=(
            'If not specified, the location stored in the property '
            ' [composer/location] will be used.'
        ),
    )
    parser.display_info.AddFormat('table[box]('
                                  'name.segment(5):label=NAME,'
                                  'name.segment(3):label=LOCATION,'
                                  'state:label=STATE,'
                                  'createTime:reverse'
                                  ')')
    parser.display_info.AddUriFunc(List._GetUri)

  def Run(self, args):
    location_refs = flags.FallthroughToLocationProperty(
        args.CONCEPTS.locations.Parse(),
        '--locations',
        'One or more locations in which to list environments must be provided.')

    return environments_api_util.List(
        location_refs,
        args.page_size,
        limit=args.limit,
        release_track=self.ReleaseTrack())



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for cluster maintenance window and maintenance exclusion window commands."""

from googlecloudsdk.api_lib.edge_cloud.container import util
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.run import flags


def RequestWithNewMaintenanceExclusion(req, messages, args):
  """Returns an update request with a new maintenance exclusion window with id, start time, and end time specified from args.

  Args:
    req: API request to be issued.
    messages: message module of edgecontainer cluster.
    args: command line arguments.

  Returns:
    modified request
  """
  if req.cluster.maintenancePolicy is None:
    req.cluster.maintenancePolicy = messages.MaintenancePolicy()
  if req.cluster.maintenancePolicy.maintenanceExclusions is None:
    req.cluster.maintenancePolicy.maintenanceExclusions = []

  req.cluster.maintenancePolicy.maintenanceExclusions.append(
      messages.MaintenanceExclusionWindow(
          id=args.add_maintenance_exclusion_name,
          window=messages.TimeWindow(
              startTime=args.add_maintenance_exclusion_start,
              endTime=args.add_maintenance_exclusion_end,
          ),
      )
  )
  return req


def AddMaintenanceExclusionWindow(ref, args, request):
  """Adds a maintenance exclusion window to the cluster if relevant flags are set.

  Args:
    ref: reference to the cluster object.
    args: command line arguments.
    request: API request to be issued

  Returns:
    modified request
  """
  del ref  # unused argument
  # If none are set, pass through original request.
  if (
      not flags.FlagIsExplicitlySet(args, "add_maintenance_exclusion_name")
      and not flags.FlagIsExplicitlySet(args, "add_maintenance_exclusion_start")
      and not flags.FlagIsExplicitlySet(args, "add_maintenance_exclusion_end")
  ):
    return request

  # If at least one of them is set, ensure all flags exist.
  _CheckAddMaintenanceExclusionFlags(args)

  release_track = args.calliope_command.ReleaseTrack()
  if request.cluster is None:
    request.cluster = util.GetMessagesModule(release_track).Cluster()

  # Ensure the MEW name doesn't already exist
  if request.cluster.maintenancePolicy:
    for mew in request.cluster.maintenancePolicy.maintenanceExclusions:
      if args.add_maintenance_exclusion_name == mew.id:
        raise exceptions.BadArgumentException(
            "--add-maintenance-exclusion-name",
            "Maintenance exclusion name " + mew.id + " already exists.",
        )

  request = RequestWithNewMaintenanceExclusion(
      request, util.GetMessagesModule(release_track), args
  )
  _AddFieldToUpdateMask("maintenancePolicy", request)
  return request


def _CheckAddMaintenanceExclusionFlags(args):
  """Confirms all necessary flags for adding an exclusion window is set.

  Args:
    args: arguments passed through gcloud command

  Raises:
    BadArgumentException specifying missing flag
  """
  if not args.add_maintenance_exclusion_name:
    raise exceptions.BadArgumentException(
        "--add-maintenance-exclusion-name",
        "Every maintenance exclusion window must have a name.",
    )
  if not args.add_maintenance_exclusion_start:
    raise exceptions.BadArgumentException(
        "--add-maintenance-exclusion-start",
        "Every maintenance exclusion window must have a start time.",
    )
  if not args.add_maintenance_exclusion_end:
    raise exceptions.BadArgumentException(
        "--add-maintenance-exclusion-end",
        "Every maintenance exclusion window must have an end time.",
    )


def RemoveMaintenanceExclusionWindow(ref, args, request):
  """Removes the cluster.maintenance_policy.maintenance_exclusion_window if --remove-maintenance-exclusion-window flag is specified.

  Args:
    ref: reference to the cluster object.
    args: command line arguments.
    request: API request to be issued

  Returns:
    modified request
  """

  del ref  # unused argument

  if not flags.FlagIsExplicitlySet(args, "remove_maintenance_exclusion_window"):
    return request

  if request.cluster is None:
    release_track = args.calliope_command.ReleaseTrack()
    request.cluster = util.GetMessagesModule(release_track).Cluster()

  if request.cluster.maintenancePolicy is None:
    _AddFieldToUpdateMask("maintenancePolicy", request)
    return request

  for idx, mew in enumerate(
      request.cluster.maintenancePolicy.maintenanceExclusions
  ):
    if mew.id == args.remove_maintenance_exclusion_window:
      i = idx
      break
  else:
    raise exceptions.BadArgumentException(
        "--remove-maintenance-exclusion-window",
        "Cannot remove a maintenance exclusion window that doesn't exist.",
    )

  del request.cluster.maintenancePolicy.maintenanceExclusions[i]

  _AddFieldToUpdateMask("maintenancePolicy", request)
  return request


def ClearMaintenanceWindow(ref, args, request):
  """Clears cluster.maintenance_policy.window in the request if --clear-maintenance-window flag is specified.

  Args:
    ref: reference to the cluster object.
    args: command line arguments.
    request: API request to be issued

  Returns:
    modified request
  """

  del ref  # unused argument

  if not flags.FlagIsExplicitlySet(args, "clear_maintenance_window"):
    return request

  if not args.clear_maintenance_window:
    raise exceptions.BadArgumentException(
        "--no-clear-maintenance-window", "The flag is not supported"
    )
  if request.cluster is None:
    release_track = args.calliope_command.ReleaseTrack()
    request.cluster = util.GetMessagesModule(release_track).Cluster()
  if request.cluster.maintenancePolicy:
    if request.cluster.maintenancePolicy.maintenanceExclusions:
      raise exceptions.BadArgumentException(
          "--clear-maintenance-window",
          "Cannot clear a maintenance window if there are maintenance"
          " exclusions.",
      )
  request.cluster.maintenancePolicy = None

  _AddFieldToUpdateMask("maintenancePolicy", request)
  return request


def _AddFieldToUpdateMask(field, request):
  if not request.updateMask:
    request.updateMask = field
    return request

  if field not in request.updateMask:
    request.updateMask = request.updateMask + "," + field
  return request



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implements command to remove an SSH public key from the OS Login profile."""


from googlecloudsdk.api_lib.oslogin import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.oslogin import oslogin_utils
from googlecloudsdk.core import properties


def _TransformExpiry(resource, undefined=None):
  display = None
  value = resource.get('value')
  if value:
    display = oslogin_utils.ConvertUsecToRfc3339(
        value.get('expirationTimeUsec'))

  return display or undefined


class List(base.ListCommand):
  """List the SSH public keys from an OS Login profile."""

  def __init__(self, *args, **kwargs):
    super(List, self).__init__(*args, **kwargs)

  @staticmethod
  def Args(parser):
    base.URI_FLAG.RemoveFromParser(parser)
    parser.display_info.AddFormat("""
          table(
            key:label=FINGERPRINT,
            expiry():label=EXPIRY
          )
        """)

    parser.display_info.AddTransforms({
        'expiry': _TransformExpiry
    })

  def Run(self, args):
    """See ssh_utils.BaseSSHCLICommand.Run."""

    oslogin_client = client.OsloginClient(self.ReleaseTrack())
    user_email = (properties.VALUES.auth.impersonate_service_account.Get()
                  or properties.VALUES.core.account.Get())

    keys = oslogin_utils.GetKeysFromProfile(user_email, oslogin_client)
    return keys

List.detailed_help = {
    'brief': 'List SSH public keys from an OS Login profile.',
    'DESCRIPTION': """
      *{command}* lists the SSH public keys in an OS Login profile. By
      default, the command only displays the fingerprints and experation
      time for the keys. Additional data can be displayed using the `--format`
      flag.
    """,
    'EXAMPLES': """
      To list the keys in your OS Login profile, run:

        $ {command}

      To show all of the SSH public key information, in YAML format, run:

        $ {command} --format=yaml
    """,
}




# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to show metadata for an operation."""


from googlecloudsdk.api_lib.composer import operations_util as operations_api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.composer import resource_args


DETAILED_HELP = {
    'EXAMPLES':
        """\
          To get details for the operation ``operation-1'', run:

            $ {command} operation-1
        """
}


class Describe(base.DescribeCommand):
  """Get details about an asynchronous operation."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    resource_args.AddOperationResourceArg(parser, 'to describe')

  def Run(self, args):
    operation_ref = args.CONCEPTS.operation.Parse()
    return operations_api_util.Get(
        operation_ref, release_track=self.ReleaseTrack())



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing operations."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.api_lib.compute import request_helper
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties


def _AllowPartialError():
  return properties.VALUES.compute.allow_partial_error.GetBool()


def AddFlags(parser, is_ga):
  """Helper function for adding flags dependant on the release track."""
  parser.display_info.AddFormat("""\
      table(
        name,
        operationType:label=TYPE,
        targetLink.scope():label=TARGET,
        operation_http_status():label=HTTP_STATUS,
        status,
        insertTime:label=TIMESTAMP
      )""")
  if is_ga:
    lister.AddMultiScopeListerFlags(
        parser, zonal=True, regional=True, global_=True)
  else:
    lister.AddBaseListerArgs(parser)
    parser.add_argument(
        '--zones',
        metavar='ZONE',
        help=('If arguments are provided, only resources from the given '
              'zones are shown. If no arguments are provided all zonal '
              'operations are shown.'),
        type=arg_parsers.ArgList())
    parser.add_argument(
        '--regions',
        metavar='REGION',
        help=('If arguments are provided, only resources from the given '
              'regions are shown. If no arguments are provided all regional '
              'operations are shown.'),
        type=arg_parsers.ArgList())
    parser.add_argument(
        '--global',
        action='store_true',
        help='If provided, all global resources are shown.',
        default=False)


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.UniverseCompatible
class List(base.ListCommand):
  """List Compute Engine operations."""

  @staticmethod
  def Args(parser):
    AddFlags(parser, True)

  def NoArguments(self, args):
    """Determine if the user provided any flags indicating scope."""
    no_compute_args = (args.zones is None and args.regions is None and
                       not getattr(args, 'global'))
    return no_compute_args

  def Run(self, args):
    """Yields zonal, regional, and/or global resources."""
    compute_holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    compute_client = compute_holder.client

    # This is True if the user provided no flags indicating scope.
    no_scope_flags = self.NoArguments(args)

    requests = []
    request_data = lister.ParseNamesAndRegexpFlags(args,
                                                   compute_holder.resources)

    # TODO(b/36050874): Start using aggregatedList for zones and regions when
    # the operations list API supports them.
    if no_scope_flags:
      request = compute_client.apitools_client.globalOperations.GetRequestType(
          'AggregatedList'
      )(
          filter=request_data.filter,
          maxResults=request_data.max_results,
          project=list(request_data.scope_set)[0].project,
      )
      if hasattr(request, 'returnPartialSuccess'):
        request.returnPartialSuccess = True
      requests.append((
          compute_client.apitools_client.globalOperations,
          'AggregatedList',
          request,
      ))
    else:
      if getattr(args, 'global'):
        requests.append(
            (compute_client.apitools_client.globalOperations, 'List',
             compute_client.apitools_client.globalOperations.GetRequestType(
                 'List')(
                     filter=request_data.filter,
                     maxResults=request_data.max_results,
                     project=list(request_data.scope_set)[0].project)))
      if args.regions is not None:
        args_region_names = [
            compute_holder.resources.Parse(  # pylint:disable=g-complex-comprehension
                region,
                params={'project': properties.VALUES.core.project.GetOrFail},
                collection='compute.regions').Name()
            for region in args.regions or []]
        # If no regions were provided by the user, fetch a list.
        errors = []
        region_names = (
            args_region_names or [res.name for res in lister.GetGlobalResources(  # pylint:disable=g-complex-comprehension
                service=compute_client.apitools_client.regions,
                project=properties.VALUES.core.project.GetOrFail(),
                filter_expr=None,
                http=compute_client.apitools_client.http,
                batch_url=compute_client.batch_url,
                errors=errors)])
        if errors:
          utils.RaiseToolException(
              errors,
              'Unable to fetch a list of regions. Specifying [--regions] may '
              'fix this issue:')
        for region_name in region_names:
          requests.append(
              (compute_client.apitools_client.regionOperations, 'List',
               compute_client.apitools_client.regionOperations.GetRequestType(
                   'List')(
                       filter=request_data.filter,
                       maxResults=request_data.max_results,
                       region=region_name,
                       project=list(request_data.scope_set)[0].project)))
      if args.zones is not None:
        args_zone_names = [
            compute_holder.resources.Parse(  # pylint:disable=g-complex-comprehension
                zone,
                params={
                    'project': properties.VALUES.core.project.GetOrFail,
                },
                collection='compute.zones').Name()
            for zone in args.zones or []]
        # If no zones were provided by the user, fetch a list.
        errors = []
        zone_names = (
            args_zone_names or [res.name for res in lister.GetGlobalResources(  # pylint:disable=g-complex-comprehension
                service=compute_client.apitools_client.zones,
                project=properties.VALUES.core.project.GetOrFail(),
                filter_expr=None,
                http=compute_client.apitools_client.http,
                batch_url=compute_client.batch_url,
                errors=errors)])
        if errors:
          utils.RaiseToolException(
              errors,
              'Unable to fetch a list of zones. Specifying [--zones] may '
              'fix this issue:')
        for zone_name in zone_names:
          requests.append(
              (compute_client.apitools_client.zoneOperations, 'List',
               compute_client.apitools_client.zoneOperations.GetRequestType(
                   'List')(
                       filter=request_data.filter,
                       maxResults=request_data.max_results,
                       zone=zone_name,
                       project=list(request_data.scope_set)[0].project)))
    errors = []
    results = list(
        request_helper.ListJson(
            requests=requests,
            http=compute_client.apitools_client.http,
            batch_url=compute_client.batch_url,
            errors=errors))

    if errors:
      if _AllowPartialError():
        utils.WarnIfPartialRequestFail(errors)
      else:
        utils.RaiseToolException(errors)

    return results


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class ListBeta(List):
  """List Compute Engine operations."""

  @staticmethod
  def Args(parser):
    AddFlags(parser, False)

List.detailed_help = base_classes.GetGlobalRegionalListerHelp('operations')
ListBeta.detailed_help = {
    'brief': 'List Compute Engine operations',
    'DESCRIPTION': """
        *{command}* displays all Compute Engine operations in a
        project.

        By default, all global, regional, and zonal operations are listed. The
        results can be narrowed by providing combinations of the --zones,
        --regions, and --global flags.

        Note: *{command}* displays operations fewer than 14 days old, up to a
        maximum of 5000.
        """,
    'EXAMPLES': """
        To list all operations in a project in table form, run:

          $ {command}

        To list the URIs of all operations in a project, run:

          $ {command} --uri

        To list all operations in zones us-central1-b and
        europe-west1-d, run:

           $ {command} --zones=us-central1-b,europe-west1-d

        To list all global operations in a project, run:

           $ {command} --global

        To list all regional operations in a project, run:

           $ {command} --regions=""

        To list all operations with names prefixed with `operation`, run:

           $ {command} --filter="name:operation"

        To list all operations in the us-central1 and europe-west1
        regions and all operations in the us-central1-a zone, run:

           $ {command} --zones=us-central1-a --regions=us-central1,europe-west1

        To list all operations with a specified target, filter on the targetLink
        field (run `{command} --format=json` to see a full, well-structured list
        of available fields). Additionally, use `scope()` which extracts the
        last part of the URL to get the required target information, and run:

           $ {command} --filter="targetLink.scope():default-12345abc"
        """,
}



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to describe a Data Fusion instance."""


from googlecloudsdk.api_lib.data_fusion import datafusion as df
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.data_fusion import resource_args


@base.RegionalEndpointsSupported
class Describe(base.DescribeCommand):
  """Gets details about a Cloud Data Fusion instance.

  ## EXAMPLES

  To describe instance `my-instance` in project `my-project` and location
  `my-location`, run:

    $ {command} my-instance --project=my-project --location=my-location
  """

  @staticmethod
  def Args(parser):
    resource_args.AddInstanceResourceArg(parser, 'Instance to describe.')
    parser.display_info.AddFormat(
        'table[box]('
        'name.segment(5):label=NAME,'
        'type:label=EDITION,'
        'createTime:reverse:label=CREATE_TIME,'
        'updateTime:reverse:label=UPDATE_TIME,'
        'zone:label=ZONE,'
        'version:label=VERSION,'
        'patchRevision:label=PATCH_REVISION,'
        'availableVersion:label=AVAILABLE_VERSIONS_TO_UPDATE,'
        'service_endpoint:label=INSTANCE_URL'
        ')'
    )

  def Run(self, args):
    instance_ref = args.CONCEPTS.instance.Parse()
    datafusion = df.Datafusion(location=instance_ref.locationsId)

    request = (
        datafusion.messages.DatafusionProjectsLocationsInstancesGetRequest(
            name=instance_ref.RelativeName()
        )
    )

    instance = datafusion.client.projects_locations_instances.Get(request)
    return instance



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1beta1.types import content


__protobuf__ = proto.module(
    package='google.cloud.aiplatform.v1beta1',
    manifest={
        'UsageMetadata',
    },
)


class UsageMetadata(proto.Message):
    r"""Usage metadata about the content generation request and
    response. This message provides a detailed breakdown of token
    usage and other relevant metrics.

    Attributes:
        prompt_token_count (int):
            The total number of tokens in the prompt. This includes any
            text, images, or other media provided in the request. When
            ``cached_content`` is set, this also includes the number of
            tokens in the cached content.
        candidates_token_count (int):
            The total number of tokens in the generated
            candidates.
        total_token_count (int):
            The total number of tokens for the entire request. This is
            the sum of ``prompt_token_count``,
            ``candidates_token_count``, ``tool_use_prompt_token_count``,
            and ``thoughts_token_count``.
        tool_use_prompt_token_count (int):
            Output only. The number of tokens in the
            results from tool executions, which are provided
            back to the model as input, if applicable.
        thoughts_token_count (int):
            Output only. The number of tokens that were
            part of the model's generated "thoughts" output,
            if applicable.
        cached_content_token_count (int):
            Output only. The number of tokens in the
            cached content that was used for this request.
        prompt_tokens_details (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1beta1.types.ModalityTokenCount]):
            Output only. A detailed breakdown of the
            token count for each modality in the prompt.
        cache_tokens_details (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1beta1.types.ModalityTokenCount]):
            Output only. A detailed breakdown of the
            token count for each modality in the cached
            content.
        candidates_tokens_details (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1beta1.types.ModalityTokenCount]):
            Output only. A detailed breakdown of the
            token count for each modality in the generated
            candidates.
        tool_use_prompt_tokens_details (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1beta1.types.ModalityTokenCount]):
            Output only. A detailed breakdown by modality
            of the token counts from the results of tool
            executions, which are provided back to the model
            as input.
        traffic_type (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1beta1.types.UsageMetadata.TrafficType):
            Output only. The traffic type for this
            request.
    """
    class TrafficType(proto.Enum):
        r"""The type of traffic that this request was processed with,
        indicating which quota gets consumed.

        Values:
            TRAFFIC_TYPE_UNSPECIFIED (0):
                Unspecified request traffic type.
            ON_DEMAND (1):
                Type for Pay-As-You-Go traffic.
            ON_DEMAND_PRIORITY (3):
                Type for Priority Pay-As-You-Go traffic.
            ON_DEMAND_FLEX (4):
                Type for Flex traffic.
            PROVISIONED_THROUGHPUT (2):
                Type for Provisioned Throughput traffic.
        """
        TRAFFIC_TYPE_UNSPECIFIED = 0
        ON_DEMAND = 1
        ON_DEMAND_PRIORITY = 3
        ON_DEMAND_FLEX = 4
        PROVISIONED_THROUGHPUT = 2

    prompt_token_count: int = proto.Field(
        proto.INT32,
        number=1,
    )
    candidates_token_count: int = proto.Field(
        proto.INT32,
        number=2,
    )
    total_token_count: int = proto.Field(
        proto.INT32,
        number=3,
    )
    tool_use_prompt_token_count: int = proto.Field(
        proto.INT32,
        number=13,
    )
    thoughts_token_count: int = proto.Field(
        proto.INT32,
        number=14,
    )
    cached_content_token_count: int = proto.Field(
        proto.INT32,
        number=5,
    )
    prompt_tokens_details: MutableSequence[content.ModalityTokenCount] = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message=content.ModalityTokenCount,
    )
    cache_tokens_details: MutableSequence[content.ModalityTokenCount] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message=content.ModalityTokenCount,
    )
    candidates_tokens_details: MutableSequence[content.ModalityTokenCount] = proto.RepeatedField(
        proto.MESSAGE,
        number=11,
        message=content.ModalityTokenCount,
    )
    tool_use_prompt_tokens_details: MutableSequence[content.ModalityTokenCount] = proto.RepeatedField(
        proto.MESSAGE,
        number=12,
        message=content.ModalityTokenCount,
    )
    traffic_type: TrafficType = proto.Field(
        proto.ENUM,
        number=8,
        enum=TrafficType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Vertex AI index endpoints update command."""


from googlecloudsdk.api_lib.ai.index_endpoints import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import errors
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import validation
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.GA)
class UpdateV1(base.UpdateCommand):
  """Update an Vertex AI index endpoint.

  ## EXAMPLES

  To update display name of index endpoint `123` under project `example` in
  region `us-central1`, run:

    $ {command} --display-name=new-name --project=example --region=us-central1
  """

  @staticmethod
  def Args(parser):
    flags.AddIndexEndpointResourceArg(parser, 'to update')
    flags.GetDisplayNameArg(
        'index endpoint', required=False).AddToParser(parser)
    flags.GetDescriptionArg('index endpoint').AddToParser(parser)
    labels_util.AddUpdateLabelsFlags(parser)

  def _Run(self, args, version):
    validation.ValidateDisplayName(args.display_name)
    index_endpoint_ref = args.CONCEPTS.index_endpoint.Parse()
    region = index_endpoint_ref.AsDict()['locationsId']
    with endpoint_util.AiplatformEndpointOverrides(version, region=region):
      index_endpoint_client = client.IndexEndpointsClient(version=version)
      try:
        if version == constants.GA_VERSION:
          result = index_endpoint_client.Patch(index_endpoint_ref, args)
        else:
          result = index_endpoint_client.PatchBeta(index_endpoint_ref, args)
      except errors.NoFieldsSpecifiedError:
        available_update_args = [
            'display_name', 'description', 'update_labels', 'clear_labels',
            'remove_labels'
        ]
        if not any(args.IsSpecified(arg) for arg in available_update_args):
          raise
        log.status.Print('No update to perform.')
        return None
      else:
        log.UpdatedResource(result.name, kind='Vertex AI index endpoint')
        return result

  def Run(self, args):
    return self._Run(args, constants.GA_VERSION)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class UpdateV1Beta1(UpdateV1):
  """Update an Vertex AI index endpoint.

  ## EXAMPLES

  To update display name of index endpoint `123` under project `example` in
  region `us-central1`, run:

    $ {command} --display-name=new-name --project=example --region=us-central1
  """

  def Run(self, args):
    return self._Run(args, constants.BETA_VERSION)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to get the public certificate of an Azure Client."""


import base64

from apitools.base.py import encoding
from googlecloudsdk.api_lib.container.gkemulticloud import azure as api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.azure import resource_args
from googlecloudsdk.command_lib.container.gkemulticloud import endpoint_util
from googlecloudsdk.command_lib.container.gkemulticloud import flags
from googlecloudsdk.core import log


_EXAMPLES = """
To get the public certificate of an Azure client named ``my-client'' in
location ``us-west1'', run:

$ {command} my-client --location=us-west1

To store the certificate in a file named ``client.crt'', run:

$ {command} my-client --location=us-west1 --output-file=client.crt
"""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class GetPublicCert(base.DescribeCommand):
  """Get the public certificate of an Azure client."""

  detailed_help = {'EXAMPLES': _EXAMPLES}

  @staticmethod
  def Args(parser):
    resource_args.AddAzureClientResourceArg(
        parser, 'to get the public certificate'
    )
    flags.AddOutputFile(parser, 'to store PEM')

  def Run(self, args):
    """Runs the get-public-cert command."""
    with endpoint_util.GkemulticloudEndpointOverride(
        resource_args.ParseAzureClientResourceArg(args).locationsId,
        self.ReleaseTrack(),
    ):
      client_ref = resource_args.ParseAzureClientResourceArg(args)
      api_client = api_util.ClientsClient()
      client = api_client.Get(client_ref)
      cert = self._GetCert(client)
      log.WriteToFileOrStdout(
          args.output_file if args.output_file else '-',
          cert,
          overwrite=True,
          binary=False,
          private=True,
      )

  def _GetCert(self, client):
    if client.pemCertificate:
      return client.pemCertificate
    # Support older versions with certificate instead of pemCertificate.
    # This is needed because "certificate" was removed from the proto so
    # the apigen tool doesn't generate it in the AzureClient class.
    client_dict = encoding.MessageToPyValue(client)
    if 'certificate' in client_dict:
      return base64.b64decode(client_dict['certificate'].encode('utf-8'))



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for adding member interconnects to an interconnect group."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.interconnects.groups import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.interconnects.groups import flags
from googlecloudsdk.core import properties

DETAILED_HELP = {
    'DESCRIPTION': """\
        *{command}* is used to add member interconnects to an interconnect
        group.

        For an example, refer to the *EXAMPLES* section below.
        """,
    'EXAMPLES': """\
        To add interconnects interconnect1 and interconnect2 to interconnect
        group example-interconnect-group, run:

          $ {command} example-interconnect-group
          --interconnects=interconnect1,interconnect2
        """,
}


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class AddMembers(base.UpdateCommand):
  """Add member interconnects to a Compute Engine interconnect group.

  *{command}* adds member interconnects to a Compute Engine interconnect group.
  """

  INTERCONNECT_GROUP_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.INTERCONNECT_GROUP_ARG = flags.InterconnectGroupArgument(plural=False)
    cls.INTERCONNECT_GROUP_ARG.AddArgument(parser, operation_type='update')
    flags.GetMemberInterconnects(parser)

  def Collection(self):
    return 'compute.interconnectGroups'

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.INTERCONNECT_GROUP_ARG.ResolveAsResource(args, holder.resources)
    project = properties.VALUES.core.project.GetOrFail()

    interconnect_group = client.InterconnectGroup(
        ref, project, compute_client=holder.client, resources=holder.resources
    )

    interconnects = set()
    interconnect_group_interconnects = (
        interconnect_group.Describe().interconnects
    )
    if interconnect_group_interconnects is not None:
      interconnects = set(
          property.key
          for property in interconnect_group_interconnects.additionalProperties
      )
    interconnects |= set(args.interconnects)

    return interconnect_group.Patch(
        interconnects=sorted(list(interconnects)),
    )


AddMembers.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the Config API."""


from apitools.base.py import encoding
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources
from googlecloudsdk.core.console import progress_tracker

_API_NAME = 'config'

# The maximum amount of time to wait in between polling long-running operations.
_WAIT_CEILING_MS = 10 * 1000

# The maximum amount of time to wait for the long-running operation.
_MAX_WAIT_TIME_MS = 3 * 60 * 60 * 1000

RELEASE_TRACK_TO_API_VERSION = {
    base.ReleaseTrack.ALPHA: 'v1alpha2',
    base.ReleaseTrack.GA: 'v1',
}


def GetMessagesModule(release_track=base.ReleaseTrack.GA):
  """Returns the messages module for Config API.

  Args:
    release_track: The desired value of the enum
      googlecloudsdk.calliope.base.ReleaseTrack.

  Returns:
    Module containing the definitions of messages for Config API.
  """
  return apis.GetMessagesModule(
      _API_NAME, RELEASE_TRACK_TO_API_VERSION[release_track]
  )


def GetClientInstance(release_track=base.ReleaseTrack.GA, use_http=True):
  """Returns an instance of the Config client.

  Args:
    release_track: The desired value of the enum
      googlecloudsdk.calliope.base.ReleaseTrack.
    use_http: bool, True to create an http object for this client.

  Returns:
    base_api.BaseApiClient, An instance of the Config client.
  """
  return apis.GetClientInstance(
      _API_NAME,
      RELEASE_TRACK_TO_API_VERSION[release_track],
      no_http=(not use_http),
  )


def GetDeployment(name):
  """Calls into the GetDeployment API.

  Args:
    name: the fully qualified name of the deployment, e.g.
      "projects/p/locations/l/deployments/d".

  Returns:
    A messages.Deployment.

  Raises:
    HttpNotFoundError: if the deployment doesn't exist.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE
  return client.projects_locations_deployments.Get(
      messages.ConfigProjectsLocationsDeploymentsGetRequest(name=name)
  )


def CreateDeployment(deployment, deployment_id, location):
  """Calls into the CreateDeployment API.

  Args:
    deployment: a messages.Deployment resource (containing properties like the
      blueprint).
    deployment_id: the ID of the deployment, e.g. "my-deployment" in
      "projects/p/locations/l/deployments/my-deployment".
    location: the location in which to create the deployment.

  Returns:
    A messages.OperationMetadata representing a long-running operation.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE
  return client.projects_locations_deployments.Create(
      messages.ConfigProjectsLocationsDeploymentsCreateRequest(
          parent=location, deployment=deployment, deploymentId=deployment_id
      )
  )


def UpdateDeployment(deployment, deployment_full_name):
  """Calls into the UpdateDeployment API.

  Args:
    deployment: a messages.Deployment resource (containing properties like the
      blueprint).
    deployment_full_name: the fully qualified name of the deployment.

  Returns:
    A messages.OperationMetadata representing a long-running operation.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE
  return client.projects_locations_deployments.Patch(
      messages.ConfigProjectsLocationsDeploymentsPatchRequest(
          deployment=deployment, name=deployment_full_name, updateMask=None
      )
  )


def WaitForApplyDeploymentOperation(
    operation, progress_message='Applying the deployment'
):
  """Waits for the given "apply deployment" LRO to complete.

  Args:
    operation: the operation to poll.
    progress_message: string to display for default progress_tracker.

  Raises:
    apitools.base.py.HttpError: if the request returns an HTTP error.

  Returns:
    A messages.Deployment resource.
  """
  client = GetClientInstance()
  operation_ref = resources.REGISTRY.ParseRelativeName(
      operation.name, collection='config.projects.locations.operations'
  )
  poller = waiter.CloudOperationPoller(
      client.projects_locations_deployments,
      client.projects_locations_operations,
  )

  poller.detailed_message = ''

  def TrackerUpdateFunc(tracker, result, unused_status):
    """Updates the progress tracker with the result of the operation.

    Args:
      tracker: The ProgressTracker for the operation.
      result: the operation poll result.
      unused_status: map of stages with key as stage key (string) and value is
        the progress_tracker.Stage.
    """
    messages = GetMessagesModule()

    # Need to encode to JSON and then decode to Message to be able to reasonably
    # access attributes.
    json_val = encoding.MessageToJson(result.metadata)
    deployment_metadata = encoding.JsonToMessage(
        messages.OperationMetadata, json_val
    ).deploymentMetadata

    logs = ''
    step = ''
    if deployment_metadata is not None:
      logs = deployment_metadata.logs
      step = deployment_metadata.step

    if logs is not None and step is None:
      poller.detailed_message = 'logs={0} '.format(logs)
    elif logs is not None and step is not None:
      poller.detailed_message = 'logs={0}, step={1} '.format(logs, step)

    tracker.Tick()

  def DetailMessageCallback():
    """Returns the detailed progress message to be updated on the progress tracker."""

    return poller.detailed_message

  aborted_message = 'Aborting wait for operation {0}.\n'.format(operation_ref)
  custom_tracker = progress_tracker.ProgressTracker(
      message=progress_message,
      detail_message_callback=DetailMessageCallback,
      aborted_message=aborted_message,
  )

  result = waiter.WaitFor(
      poller,
      operation_ref,
      progress_message,
      custom_tracker=custom_tracker,
      tracker_update_func=TrackerUpdateFunc,
      max_wait_ms=_MAX_WAIT_TIME_MS,
      wait_ceiling_ms=_WAIT_CEILING_MS,
  )
  return result


def ImportStateFile(import_state_file_request, deployment_id):
  """Calls ImportStateFile API.

  Args:
    import_state_file_request: a messages.ImportStateFileRequest.
    deployment_id: the ID of the deployment, e.g. "my-deployment" in
      "projects/p/locations/l/deployments/my-deployment".

  Returns:
    (Statefile) The response message.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE

  return client.projects_locations_deployments.ImportState(
      messages.ConfigProjectsLocationsDeploymentsImportStateRequest(
          importStatefileRequest=import_state_file_request, parent=deployment_id
      )
  )


def ExportDeploymentStateFile(
    export_deployment_state_file_request,
    deployment_id,
):
  """Calls ExportDeploymentStateFile API.

  Args:
    export_deployment_state_file_request: A ExportDeploymentStatefileRequest
      resource to be passed as the request body.
    deployment_id: the ID of the deployment, e.g. "my-deployment" in
      "projects/p/locations/l/deployments/my-deployment".

  Returns:
    (Statefile) The response message.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE

  return client.projects_locations_deployments.ExportState(
      messages.ConfigProjectsLocationsDeploymentsExportStateRequest(
          exportDeploymentStatefileRequest=export_deployment_state_file_request,
          parent=deployment_id,
      )
  )


def ExportRevisionStateFile(
    export_revision_state_file_request,
    deployment_id,
):
  """Calls ExportDeploymentRevisionsStateFile API.

  Args:
    export_revision_state_file_request: A ExportRevisionStatefileRequest
      resource to be passed as the request body.
    deployment_id: the ID of the deployment, e.g. "my-deployment" in
      "projects/p/locations/l/deployments/my-deployment".

  Returns:
    (Statefile) The response message.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE

  return client.projects_locations_deployments_revisions.ExportState(
      messages.ConfigProjectsLocationsDeploymentsRevisionsExportStateRequest(
          exportRevisionStatefileRequest=export_revision_state_file_request,
          parent=deployment_id,
      )
  )


def ExportLock(deployment_full_name):
  """Calls ExportLock API.

  Args:
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".

  Returns:
    A lock info response.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE

  return client.projects_locations_deployments.ExportLock(
      messages.ConfigProjectsLocationsDeploymentsExportLockRequest(
          name=deployment_full_name,
      )
  )


def LockDeployment(
    lock_deployment_request,
    deployment_full_name,
):
  """Calls deployment Lock API.

  Args:
    lock_deployment_request: A LockDeploymentRequest resource to be passed as
      the request body
    deployment_full_name: the ID of the deployment, e.g. "my-deployment" in
      "projects/p/locations/l/deployments/my-deployment".

  Returns:
    A long running operation.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE

  return client.projects_locations_deployments.Lock(
      messages.ConfigProjectsLocationsDeploymentsLockRequest(
          lockDeploymentRequest=lock_deployment_request,
          name=deployment_full_name,
      )
  )


def UnlockDeployment(
    unlock_deployment_request,
    deployment_full_name,
):
  """Calls deployment Unlock API.

  Args:
    unlock_deployment_request: A UnlockDeploymentRequest resource to be passed
      as the request body
    deployment_full_name: the ID of the deployment, e.g. "my-deployment" in
      format "projects/p/locations/l/deployments/my-deployment".

  Returns:
    A long running operation.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE

  return client.projects_locations_deployments.Unlock(
      messages.ConfigProjectsLocationsDeploymentsUnlockRequest(
          unlockDeploymentRequest=unlock_deployment_request,
          name=deployment_full_name,
      )
  )


def ListRevisions(deployment_full_name):
  """Lists all revisions for a deployment.

  Args:
    deployment_full_name: the fully qualified name of the deployment, e.g.
      "projects/p/locations/l/deployments/d".

  Returns:
    (ListRevisionsResponse) The response message.

  Raises:
    HttpNotFoundError: if the deployment doesn't exist.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE
  return client.projects_locations_deployments_revisions.List(
      messages.ConfigProjectsLocationsDeploymentsRevisionsListRequest(
          parent=deployment_full_name
      )
  )


def ExportPreviewResult(
    export_preview_result_request,
    preview_id,
):
  """Calls ExportPreviewResult API.

  Args:
    export_preview_result_request: A ExportPreviewResultRequest
      resource to be passed as the request body.
    preview_id: the ID of the preview, e.g. "my-preview" in
      "projects/p/locations/l/previews/my-preview".

  Returns:
    (Statefile) The response message.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE

  return client.projects_locations_previews.Export(
      messages.ConfigProjectsLocationsPreviewsExportRequest(
          exportPreviewResultRequest=export_preview_result_request,
          parent=preview_id,
      )
  )


def CreatePreview(preview, preview_id, location):
  """Calls into the CreatePreview API.

  Args:
    preview: a messages.Preview resource (containing properties like the
      blueprint).
    preview_id: the ID of the preview, e.g. "my-preview" in
      "projects/p/locations/l/previews/my-preview".
    location: the location in which to create the preview.

  Returns:
    A messages.OperationMetadata representing a long-running operation.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE
  return client.projects_locations_previews.Create(
      messages.ConfigProjectsLocationsPreviewsCreateRequest(
          parent=location, preview=preview, previewId=preview_id
      )
  )


def WaitForCreatePreviewOperation(
    operation, progress_message='Creating the preview'
):
  """Waits for the given "create preview" LRO to complete.

  Args:
    operation: the operation to poll.
    progress_message: string to display for default progress_tracker.

  Raises:
    apitools.base.py.HttpError: if the request returns an HTTP error.

  Returns:
    A messages.Preview resource.
  """
  client = GetClientInstance()
  operation_ref = resources.REGISTRY.ParseRelativeName(
      operation.name, collection='config.projects.locations.operations'
  )
  poller = waiter.CloudOperationPoller(
      client.projects_locations_previews,
      client.projects_locations_operations,
  )

  poller.detailed_message = ''

  def TrackerUpdateFunc(tracker, result, unused_status):
    """Updates the progress tracker with the result of the operation.

    Args:
      tracker: The ProgressTracker for the operation.
      result: the operation poll result.
      unused_status: map of stages with key as stage key (string) and value is
        the progress_tracker.Stage.
    """
    messages = GetMessagesModule()

    # Need to encode to JSON and then decode to Message to be able to reasonably
    # access attributes.
    json_val = encoding.MessageToJson(result.metadata)
    preview_metadata = encoding.JsonToMessage(
        messages.OperationMetadata, json_val
    ).previewMetadata

    logs = ''
    step = ''
    if preview_metadata is not None:
      logs = preview_metadata.logs
      step = preview_metadata.step

    if logs is not None and step is None:
      poller.detailed_message = 'logs={0} '.format(logs)
    elif logs is not None and step is not None:
      poller.detailed_message = 'logs={0}, step={1} '.format(logs, step)

    tracker.Tick()

  def DetailMessageCallback():
    """Returns the detailed progress message to be updated on the progress tracker."""

    return poller.detailed_message

  aborted_message = 'Aborting wait for operation {0}.\n'.format(operation_ref)
  custom_tracker = progress_tracker.ProgressTracker(
      message=progress_message,
      detail_message_callback=DetailMessageCallback,
      aborted_message=aborted_message,
  )

  result = waiter.WaitFor(
      poller,
      operation_ref,
      progress_message,
      custom_tracker=custom_tracker,
      tracker_update_func=TrackerUpdateFunc,
      max_wait_ms=_MAX_WAIT_TIME_MS,
      wait_ceiling_ms=_WAIT_CEILING_MS,
  )
  return result


def GetLocation(args):
  """Gets location resource from either argument flag or attribute.

  Args:
    args: The command-line arguments.

  Returns:
    The location string.
  """
  return args.location or properties.VALUES.inframanager.location.GetOrFail()


def GetProject(args):
  """Gets project resource from either argument flag or attribute.

  Args:
    args: The command-line arguments.

  Returns:
    The project string.
  """
  return args.project or properties.VALUES.core.project.GetOrFail()


def GetAutoMigrationConfigName(project, location):
  """Returns the name of the auto migration config."""
  return f'projects/{project}/locations/{location}/autoMigrationConfig'


def SetDisableAutoMigrationInRequest(ref, args, req):
  """Sets fields in AutoMigrationConfig request to disable the config."""
  del ref  # Unused.
  messages = GetMessagesModule()
  project = GetProject(args)
  location = GetLocation(args)
  config_name = GetAutoMigrationConfigName(project, location)
  req.name = config_name
  req.autoMigrationConfig = messages.AutoMigrationConfig(
      name=config_name, autoMigrationEnabled=False
  )
  req.updateMask = 'auto_migration_enabled'
  return req


def SetGetAutoMigrationInRequest(ref, args, req):
  """Sets fields in AutoMigrationConfig request to get the config."""
  del ref  # Unused.
  project = GetProject(args)
  location = GetLocation(args)
  req.name = GetAutoMigrationConfigName(project, location)
  return req


def SetEnableAutoMigrationInRequest(ref, args, req):
  """Sets fields in AutoMigrationConfig request to enable the config."""
  del ref  # Unused.
  messages = GetMessagesModule()
  project = GetProject(args)
  location = GetLocation(args)
  config_name = GetAutoMigrationConfigName(project, location)
  req.name = config_name
  req.autoMigrationConfig = messages.AutoMigrationConfig(
      name=config_name, autoMigrationEnabled=True
  )
  req.updateMask = 'auto_migration_enabled'
  return req


def GetTfVersion(name):
  """Calls into the GetTerraformVersion API.

  Args:
    name: the fully qualified name of the terraform version, e.g.
      "projects/p/locations/l/terraformVersions/1.2.3".

  Returns:
    A messages.TerraformVersion.

  Raises:
    HttpNotFoundError: if the terraform version doesn't exist.
  """
  client = GetClientInstance()
  messages = client.MESSAGES_MODULE
  return client.projects_locations_terraformVersions.Get(
      messages.ConfigProjectsLocationsTerraformVersionsGetRequest(name=name)
  )



"""Generated client library for faulttesting version v1alpha."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.faulttesting.v1alpha import faulttesting_v1alpha_messages as messages


class FaulttestingV1alpha(base_api.BaseApiClient):
  """Generated client library for service faulttesting version v1alpha."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://faulttesting.googleapis.com/'
  MTLS_BASE_URL = 'https://faulttesting.mtls.googleapis.com/'

  _PACKAGE = 'faulttesting'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'FaulttestingV1alpha'
  _URL_VERSION = 'v1alpha'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new faulttesting handle."""
    url = url or self.BASE_URL
    super(FaulttestingV1alpha, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_experimentTemplates = self.ProjectsLocationsExperimentTemplatesService(self)
    self.projects_locations_experiments_affectedResources = self.ProjectsLocationsExperimentsAffectedResourcesService(self)
    self.projects_locations_experiments = self.ProjectsLocationsExperimentsService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_validations_validationResources = self.ProjectsLocationsValidationsValidationResourcesService(self)
    self.projects_locations_validations = self.ProjectsLocationsValidationsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsExperimentTemplatesService(base_api.BaseApiService):
    """Service class for the projects_locations_experimentTemplates resource."""

    _NAME = 'projects_locations_experimentTemplates'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsExperimentTemplatesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""CreateExperimentTemplate creates a new experiment template.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExperimentTemplate) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates',
        http_method='POST',
        method_id='faulttesting.projects.locations.experimentTemplates.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['experimentTemplateId'],
        relative_path='v1alpha/{+parent}/experimentTemplates',
        request_field='experimentTemplate',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesCreateRequest',
        response_type_name='ExperimentTemplate',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""DeleteExperimentTemplate deletes an experiment template.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates/{experimentTemplatesId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.experimentTemplates.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""GetExperimentTemplate gets an experiment template.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExperimentTemplate) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates/{experimentTemplatesId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.experimentTemplates.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesGetRequest',
        response_type_name='ExperimentTemplate',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""ListExperimentTemplates lists all experiment templates in a project.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListExperimentTemplatesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates',
        http_method='GET',
        method_id='faulttesting.projects.locations.experimentTemplates.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/experimentTemplates',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesListRequest',
        response_type_name='ListExperimentTemplatesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""UpdateExperimentTemplate updates an experiment template.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExperimentTemplate) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates/{experimentTemplatesId}',
        http_method='PATCH',
        method_id='faulttesting.projects.locations.experimentTemplates.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='experimentTemplate',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesPatchRequest',
        response_type_name='ExperimentTemplate',
        supports_download=False,
    )

  class ProjectsLocationsExperimentsAffectedResourcesService(base_api.BaseApiService):
    """Service class for the projects_locations_experiments_affectedResources resource."""

    _NAME = 'projects_locations_experiments_affectedResources'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsExperimentsAffectedResourcesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the details of an affected resource.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsAffectedResourcesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AffectedResource) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}/affectedResources/{affectedResourcesId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.affectedResources.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsAffectedResourcesGetRequest',
        response_type_name='AffectedResource',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List all the affected resources for a given experiment.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsAffectedResourcesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListAffectedResourcesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}/affectedResources',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.affectedResources.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/affectedResources',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsAffectedResourcesListRequest',
        response_type_name='ListAffectedResourcesResponse',
        supports_download=False,
    )

  class ProjectsLocationsExperimentsService(base_api.BaseApiService):
    """Service class for the projects_locations_experiments resource."""

    _NAME = 'projects_locations_experiments'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsExperimentsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""CreateExperiment creates a new experiment. If experiment.experiment_template is specified, a regular experiment is created. If experiment.action is specified, a template-validation experiment is created. Either way, the initial state will be PREPARING. If it's a regular Experiment, when the system is finished preparing it, the state will change to PREPARED, after which it can be started. If it's a template validation Experiment, the state will go directly from PREPARING to COMPLETED.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['experimentId'],
        relative_path='v1alpha/{+parent}/experiments',
        request_field='experiment',
        request_type_name='FaulttestingProjectsLocationsExperimentsCreateRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""DeleteExperiment deletes an experiment. Running experiments must be stopped first.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.experiments.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""GetExperiment gets an experiment.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsGetRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""ListExperiments lists all experiments in a project.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListExperimentsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/experiments',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsListRequest',
        response_type_name='ListExperimentsResponse',
        supports_download=False,
    )

    def Start(self, request, global_params=None):
      r"""StartExperiment starts an existing experiment. The experiment must already be in the PREPARED state. On return, the new state will be INJECTING. When injection is complete, the state becomes INJECTED.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsStartRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Start')
      return self._RunMethod(
          config, request, global_params=global_params)

    Start.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}:start',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.start',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:start',
        request_field='startExperimentRequest',
        request_type_name='FaulttestingProjectsLocationsExperimentsStartRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def Stop(self, request, global_params=None):
      r"""StopExperiment stops a running experiment. On return, the new state will be REVERTING. Once the revert process completes it will become COMPLETED.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsStopRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Stop')
      return self._RunMethod(
          config, request, global_params=global_params)

    Stop.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}:stop',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.stop',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:stop',
        request_field='stopExperimentRequest',
        request_type_name='FaulttestingProjectsLocationsExperimentsStopRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def StopAll(self, request, global_params=None):
      r"""StopAllExperiments stops all running experiments for a given project and location. On return, the process of stopping all experiments will have begun, but may not be completed. Call ListExperiments with filter="state!=COMPLETED" to check the progress. Returns an RPC error if it can't list experiments at all. Otherwise, it lists and stops all experiments it can, and returns a response including success count and a list of errors. If there are no experiments to stop, it returns successfully with an empty response message.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsStopAllRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (StopAllExperimentsResponse) The response message.
      """
      config = self.GetMethodConfig('StopAll')
      return self._RunMethod(
          config, request, global_params=global_params)

    StopAll.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments:stopAll',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.stopAll',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1alpha/{+parent}/experiments:stopAll',
        request_field='stopAllExperimentsRequest',
        request_type_name='FaulttestingProjectsLocationsExperimentsStopAllRequest',
        response_type_name='StopAllExperimentsResponse',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (FaulttestingProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='faulttesting.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='FaulttestingProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (FaulttestingProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (FaulttestingProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (FaulttestingProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='faulttesting.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1alpha/{+name}/operations',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsValidationsValidationResourcesService(base_api.BaseApiService):
    """Service class for the projects_locations_validations_validationResources resource."""

    _NAME = 'projects_locations_validations_validationResources'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsValidationsValidationResourcesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the details of a ValidationResource.

      Args:
        request: (FaulttestingProjectsLocationsValidationsValidationResourcesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ValidationResource) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}/validationResources/{validationResourcesId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.validationResources.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsValidationResourcesGetRequest',
        response_type_name='ValidationResource',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List all the validation resources for a given Validation.

      Args:
        request: (FaulttestingProjectsLocationsValidationsValidationResourcesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListValidationResourcesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}/validationResources',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.validationResources.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/validationResources',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsValidationResourcesListRequest',
        response_type_name='ListValidationResourcesResponse',
        supports_download=False,
    )

  class ProjectsLocationsValidationsService(base_api.BaseApiService):
    """Service class for the projects_locations_validations resource."""

    _NAME = 'projects_locations_validations'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsValidationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""CreateValidation creates a new template validation. The initial state will be PREPARING. When finished, it will change to COMPLETED.

      Args:
        request: (FaulttestingProjectsLocationsValidationsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Validation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations',
        http_method='POST',
        method_id='faulttesting.projects.locations.validations.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['validationId'],
        relative_path='v1alpha/{+parent}/validations',
        request_field='validation',
        request_type_name='FaulttestingProjectsLocationsValidationsCreateRequest',
        response_type_name='Validation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""DeleteValidation deletes a Validation.

      Args:
        request: (FaulttestingProjectsLocationsValidationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.validations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""GetValidation gets a Validation.

      Args:
        request: (FaulttestingProjectsLocationsValidationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Validation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsGetRequest',
        response_type_name='Validation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""ListValidations lists all Validations in a project.

      Args:
        request: (FaulttestingProjectsLocationsValidationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListValidationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/validations',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsListRequest',
        response_type_name='ListValidationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (FaulttestingProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the [ListLocationsRequest.name] field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (FaulttestingProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations',
        http_method='GET',
        method_id='faulttesting.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/locations',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting organization security policy association."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.org_security_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.org_security_policies import flags
from googlecloudsdk.command_lib.compute.org_security_policies import org_security_policies_utils
import six


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Delete(base.DeleteCommand):
  """Delete a Compute Engine organization security policy association.

  *{command}* is used to delete organization security policy association.
  """

  ORG_SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.ORG_SECURITY_POLICY_ARG = flags.OrgSecurityPolicyAssociationsArgument(
        required=True)
    cls.ORG_SECURITY_POLICY_ARG.AddArgument(parser, operation_type='delete')
    flags.AddArgsDeleteAssociation(parser)
    parser.display_info.AddCacheUpdater(flags.OrgSecurityPoliciesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.ORG_SECURITY_POLICY_ARG.ResolveAsResource(
        args, holder.resources, with_project=False)
    org_security_policy = client.OrgSecurityPolicy(
        ref=ref,
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower())
    security_policy_id = org_security_policies_utils.GetSecurityPolicyId(
        org_security_policy,
        args.security_policy,
        organization=args.organization)
    return org_security_policy.DeleteAssociation(
        security_policy_id=security_policy_id, only_generate_request=False)


Delete.detailed_help = {
    'EXAMPLES':
        """\
    To delete an association with name ``example-association'' of an organization
    security policy with ID ``123456789'', run:

      $ {command} example-association --security-policy=123456789
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing organization security policy rules."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import org_security_policy_rule_utils as rule_utils
from googlecloudsdk.api_lib.compute.org_security_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.org_security_policies import flags
from googlecloudsdk.command_lib.compute.org_security_policies import org_security_policies_utils
import six


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Describe(base.DescribeCommand):
  """Describe a Compute Engine organization security policy rule.

  *{command}* is used to describe organization security policy rule.
  """

  ORG_SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.ORG_SECURITY_POLICY_ARG = flags.OrgSecurityPolicyRuleArgument(
        required=True, operation="describe")
    cls.ORG_SECURITY_POLICY_ARG.AddArgument(parser)
    flags.AddOrganization(parser, required=False)
    flags.AddSecurityPolicyId(parser, operation="described")

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.ORG_SECURITY_POLICY_ARG.ResolveAsResource(
        args, holder.resources, with_project=False)
    security_policy_rule_client = client.OrgSecurityPolicyRule(
        ref=ref,
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower())
    security_policy_id = org_security_policies_utils.GetSecurityPolicyId(
        security_policy_rule_client,
        args.security_policy,
        organization=args.organization)
    return security_policy_rule_client.Describe(
        priority=rule_utils.ConvertPriorityToInt(ref.Name()),
        security_policy_id=security_policy_id,
        only_generate_request=False)


Describe.detailed_help = {
    "EXAMPLES":
        """\
    To describe a rule with priority ``10'' in an organization security policy
    with ID ``123456789'', run:

      $ {command} 10 --security-policy=123456789
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Restarts an AlloyDB instance."""


from googlecloudsdk.api_lib.alloydb import api_util
from googlecloudsdk.api_lib.alloydb import instance_operations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.alloydb import flags
from googlecloudsdk.command_lib.alloydb import instance_helper
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


# TODO(b/312466999): Change @base.DefaultUniverseOnly to
# @base.UniverseCompatible once b/312466999 is fixed.
# See go/gcloud-cli-running-tpc-tests
@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.GA, base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA
)
class Restart(base.SilentCommand):
  """Restarts an AlloyDB instance within a given cluster."""

  detailed_help = {
      'DESCRIPTION': '{description}',
      'EXAMPLES': """\
        To restart an instance, run:

          $ {command} my-instance --cluster=my-cluster --region=us-central1
        """,
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser, Parser object for command line inputs
    """
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddCluster(parser, False)
    flags.AddInstance(parser)
    flags.AddRegion(parser)
    flags.AddNodeIds(parser)

  def ConstructRestartRequest(self, **kwargs):
    return instance_helper.ConstructRestartRequestFromArgs(
        kwargs.get('alloydb_messages'),
        kwargs.get('project_ref'),
        kwargs.get('args'),
    )

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    client = api_util.AlloyDBClient(self.ReleaseTrack())
    alloydb_client = client.alloydb_client
    alloydb_messages = client.alloydb_messages
    project_ref = client.resource_parser.Create(
        'alloydb.projects.locations.clusters.instances',
        projectsId=properties.VALUES.core.project.GetOrFail,
        locationsId=args.region,
        clustersId=args.cluster,
        instancesId=args.instance,
    )
    req = self.ConstructRestartRequest(
        alloydb_messages=alloydb_messages,
        project_ref=project_ref,
        args=args
    )

    op = alloydb_client.projects_locations_clusters_instances.Restart(req)
    op_ref = resources.REGISTRY.ParseRelativeName(
        op.name, collection='alloydb.projects.locations.operations'
    )
    log.status.Print('Operation ID: {}'.format(op_ref.Name()))
    if not args.async_:
      instance_operations.Await(
          op_ref, 'Restarting instance', self.ReleaseTrack(), False
      )
    return op



# -*- coding: utf-8 -*- # Lint as: python3
# Copyright 2021 Google Inc. All Rights Reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list all Apigee archive deployments in an environment."""


from googlecloudsdk.api_lib import apigee
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.apigee import archives as archive_helper
from googlecloudsdk.command_lib.apigee import defaults
from googlecloudsdk.command_lib.apigee import resource_args


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class List(base.ListCommand):
  """List Apigee archive deployments."""

  detailed_help = {
      "EXAMPLES":
          """\
  To list all archive deployments, in an environment called ``my-env'', for the
  active Cloud Platform project, run:

      $ {command} --environment=my-env

  To list all archive deployments, for an environment named ``my-env'', in an
  organization called ``my-org'', run:

      $ {command} --environment=my-env --organization=my-org

  To list all archive deployments formatted as a JSON array, run:

      $ {command} --environment=my-env --format=json
  """
  }

  @staticmethod
  def Args(parser):
    resource_args.AddSingleResourceArgument(
        parser,
        "organization.environment",
        "Apigee environment whose archive deployments should be listed.",
        positional=False,
        required=True,
        fallthroughs=[defaults.GCPProductOrganizationFallthrough()])
    # The response is a JSON array of archive deployment descriptors, so the
    # array needs to be flattened to display as a table.
    parser.display_info.AddFlatten(["archiveDeployments[]"])
    # Cloud SDK projections can be used to format each column of the display
    # table:
    # https://cloud.google.com/sdk/gcloud/reference/topic/projections
    # The "ARCHIVE ID" column scopes the resource path in the "name" field of
    # the API response to only show the id of the archive deployment.
    archive_id_col = ("archiveDeployments.name.scope(archiveDeployments)"
                      ":label='ARCHIVE ID'")
    # The "ENVIORNMENT" column scopes the resource path in the "name" field of
    # the API response to only show the Apigee environment id.
    env_id_col = ("archiveDeployments.name.scope(environments).segment(0)"
                  ":label=ENVIRONMENT")
    # The "CREATED AT" column formats the posix epoch in the "createdAt" field
    # of the API response into a human-readable date format.
    created_col = ("archiveDeployments.createdAt.date("
                   "format='%Y-%m-%d %H:%M:%S %Z', unit=1000000, tz=LOCAL)"
                   ":label='DEPLOYED AT'")
    # The labels field is a list of key/value pairs so it is flattened to
    # display in the table.
    labels_col = "archiveDeployments.labels.flatten()"

    # The status column uses operation metadata and timestamps to determine
    # the current status of the archive deployment.
    status_col = ("archiveDeployments.operationStatus:label='OPERATION STATUS'")
    cols = ", ".join(
        [archive_id_col, env_id_col, created_col, labels_col, status_col])
    # Format the column definitions into a table.
    table_fmt = "table({})".format(cols)
    parser.display_info.AddFormat(table_fmt)

  def Run(self, args):
    """Run the list command."""
    identifiers = args.CONCEPTS.environment.Parse().AsDict()
    org = identifiers["organizationsId"]

    archive_response = apigee.ArchivesClient.List(identifiers)

    extended_archives = archive_helper.ListArchives(org).ExtendedArchives(
        archive_response)

    return {"archiveDeployments": extended_archives}



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to get a model in Vertex AI."""


from googlecloudsdk.api_lib.ai.models import client
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import region_util


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.UniverseCompatible
class DescribeV1(base.DescribeCommand):
  """Get detailed model information about the given model id.

  ## EXAMPLES

  Describe a model `123` of project `example` in region `us-central1`,
  run:

    $ {command} 123 --project=example --region=us-central1

  Describe a model `123` of version `2` of project `example` in region
  `us-central1`, run:

    $ {command} 123@2 --project=example --region=us-central1
  """

  @staticmethod
  def Args(parser):
    flags.AddModelResourceArg(parser, 'to describe',
                              region_util.PromptForOpRegion)

  def _Run(self, args, model_ref, region):
    with endpoint_util.AiplatformEndpointOverrides(
        version=constants.GA_VERSION, region=region):
      client_instance = apis.GetClientInstance(
          constants.AI_PLATFORM_API_NAME,
          constants.AI_PLATFORM_API_VERSION[constants.GA_VERSION])
      return client.ModelsClient(
          client=client_instance,
          messages=client_instance.MESSAGES_MODULE).Get(model_ref)

  def Run(self, args):
    model_ref = args.CONCEPTS.model.Parse()
    region = model_ref.AsDict()['locationsId']
    return self._Run(args, model_ref, region)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
@base.UniverseCompatible
class DescribeV1Beta1(DescribeV1):
  """Get detailed model information about the given model id.

  ## EXAMPLES

  Describe a model `123` of project `example` in region `us-central1`,
  run:

    $ {command} 123 --project=example --region=us-central1

  Describe a model `123` of version `2` of project `example` in region
  `us-central1`, run:

    $ {command} 123@2 --project=example --region=us-central1
  """

  def _Run(self, args, model_ref, region):
    with endpoint_util.AiplatformEndpointOverrides(
        version=constants.BETA_VERSION, region=region):
      response = client.ModelsClient().Get(model_ref)
      return response



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Fetch default Config Connector identity."""


import io

from googlecloudsdk.api_lib.container import util as container_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.composer import util as composer_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


def _BaseRun(args):
  """Base operations for `get-config-connector-identity` run command."""
  container_util.CheckKubectlInstalled()

  cluster_id = 'krmapihost-' + args.name
  location = args.location
  project_id = args.project or properties.VALUES.core.project.GetOrFail()

  GetConfigConnectorIdentityForCluster(location, cluster_id, project_id)


def GetConfigConnectorIdentityForCluster(location, cluster_id, project_id):
  """Get Config Connector identity for the given cluster."""
  with composer_util.TemporaryKubeconfig(location, cluster_id):
    output = io.StringIO()
    composer_util.RunKubectlCommand([
        'get', 'ConfigConnectorContext', '-o',
        'jsonpath="{.items[0].spec.googleServiceAccount}"'
    ],
                                    out_func=output.write,
                                    err_func=log.err.write,
                                    namespace='config-control')
    identity = output.getvalue().replace('"', '')
    log.status.Print(
        'Default Config Connector identity: [{identity}].\n'
        '\n'
        'For example, to give Config Connector permission to manage Google Cloud resources in the same project:\n'
        'gcloud projects add-iam-policy-binding {project_id} \\\n'
        '    --member \"serviceAccount:{identity}\" \\\n'
        '    --role \"roles/owner\" \\\n'
        '    --project {project_id}\n'.format(
            identity=identity, project_id=project_id))


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class GetConfigConnectorIdentity(base.Command):
  """Fetch default Config Connector identity.

  {command} prints the default Config Connector Google Service Account in
  a specific Anthos Config Controller.
  """
  detailed_help = {
      'EXAMPLES':
          """\
          To print the default Config Connector identity used by your
          Config Controller 'main' in the location 'us-central1', run:

            $ {command} main --location=us-central1
      """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument('name', help='Name of the Anthos Config Controller.')
    parser.add_argument(
        '--location',
        required=True,
        help='The location (region) of the Anthos Config Controller.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.
    """
    _BaseRun(args)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""`gcloud app repair` command."""


from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.calliope import base
from googlecloudsdk.core.console import progress_tracker


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class Repair(base.Command):
  """Restores required App Engine features to the current application.

  For example, this command will restore the App Engine staging bucket if it
  has been deleted. It will no longer restore the service account, instead, the
  IAM service account undelete API must be used for the purpose.
  """

  detailed_help = {
      'EXAMPLES': """\
          To repair the application, run

              $ {command}
          """,
  }

  def Run(self, args):
    api_client = appengine_api_client.AppengineApiClient.GetApiClient('v1beta')

    with progress_tracker.ProgressTracker(
        'Repairing the app [{0}]'.format(api_client.project)):
      api_client.RepairApplication()






# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for listing reservation sub-blocks."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import request_helper
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.reservations import resource_args
from googlecloudsdk.command_lib.compute.reservations.sub_blocks import flags

DETAILED_HELP = {
    'DESCRIPTION':
        """\
        {command} displays all Compute Engine reservation sub-blocks in an extended reservation's block.
      """,
    'EXAMPLES':
        """\
        To list all reservation sub-blocks in an extended reservation's block my-block in table form,
        run:

        $ {command} extended-reservation --block-name=my-block --zone=us-central1-a --project=my-project

        To list the URIs of all reservation blocks in an extended reservation, run:

        $ {command} extended-reservation --block-name=my-block --zone=us-central1-a --project=my-project --uri
    """
}


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class List(base.ListCommand):
  """List Compute Engine reservation sub-blocks."""

  detailed_help = DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    List.ReservationArg = resource_args.GetReservationResourceArg()
    List.ReservationArg.AddArgument(parser, operation_type='list')
    flags.AddListFlags(parser)

  def _Run(self, args, holder):
    client = holder.client

    reservation_ref = (
        List.ReservationArg.ResolveAsResource(
            args,
            holder.resources,
            default_scope=compute_scope.ScopeEnum.ZONE,
            scope_lister=compute_flags.GetDefaultScopeLister(client)))

    parent_name = f'reservations/{reservation_ref.reservation}/reservationBlocks/{args.block_name}'

    service = client.apitools_client.reservationSubBlocks
    request = (client.messages.
               ComputeReservationSubBlocksListRequest(
                   parentName=parent_name,
                   zone=reservation_ref.zone,
                   project=reservation_ref.project))

    errors = []
    results = list(request_helper.MakeRequests(
        requests=[(service, 'List', request)],
        http=client.apitools_client.http,
        batch_url=client.batch_url,
        errors=errors))

    if errors:
      utils.RaiseToolException(errors)
    return results

  def Run(self, args):
    """Creates and issues a reservationSubBlocks.list request.

    Args:
      args: the argparse arguments that this command was invoked with.

    Returns:
      List of reservation sub-blocks.
    """
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    return self._Run(args, holder)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of create command for making buckets."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import errors
from googlecloudsdk.command_lib.storage import errors_util
from googlecloudsdk.command_lib.storage import flags
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage import user_request_args_factory
from googlecloudsdk.command_lib.storage.resources import resource_reference
from googlecloudsdk.command_lib.storage.tasks.buckets import create_bucket_task

_LIFECYCLE_HELP_TEXT = """
Sets the lifecycle management configuration on a bucket. For example,
The following lifecycle management configuration JSON document
specifies that all objects in this bucket that are more than 365 days
old are deleted automatically:

  {
    "rule":
    [
      {
        "action": {"type": "Delete"},
        "condition": {"age": 365}
      }
    ]
  }
"""


@base.UniverseCompatible
class Create(base.Command):
  """Create buckets for storing objects."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      'DESCRIPTION': """
      Create new buckets.
      """,
      'EXAMPLES': """

      The following command creates 2 Cloud Storage buckets, one named
      ``my-bucket'' and a second bucket named ``my-other-bucket'':

        $ {command} gs://my-bucket gs://my-other-bucket

      The following command creates a bucket with the ``nearline'' default
      [storage class](https://cloud.google.com/storage/docs/storage-classes) in
      the ``asia'' [location](https://cloud.google.com/storage/docs/locations):

        $ {command} gs://my-bucket --default-storage-class=nearline --location=asia
      """,
  }

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        'url', type=str, nargs='+', help='The URLs of the buckets to create.'
    )
    parser.add_argument(
        '--location',
        '-l',
        type=str,
        required=arg_parsers.ArgRequiredInUniverse(
            default_universe=False, non_default_universe=True
        ),
        help=(
            '[Location](https://cloud.google.com/storage/docs/locations)'
            ' for the bucket. If not specified, the location used by Cloud'
            " Storage is ``us''. A bucket's location cannot be changed"
            ' after creation.'
        ),
    )
    parser.add_argument(
        '--public-access-prevention',
        '--pap',
        action=arg_parsers.StoreTrueFalseAction,
        help=(
            'Sets public access prevention to "enforced".'
            ' For details on how exactly public access is blocked, see:'
            ' http://cloud.google.com/storage/docs/public-access-prevention'
        ),
    )
    parser.add_argument(
        '--uniform-bucket-level-access',
        '-b',
        action=arg_parsers.StoreTrueFalseAction,
        help='Turns on uniform bucket-level access setting. Default is False.',
    )
    parser.add_argument(
        '--default-storage-class',
        '-c',
        '-s',
        type=str,
        help=(
            'Default [storage class]'
            '(https://cloud.google.com/storage/docs/storage-classes) for'
            ' the bucket. If not specified, the default storage class'
            ' used by Cloud Storage is "Standard".'
        ),
    )
    parser.add_argument(
        '--default-encryption-key',
        '-k',
        type=str,
        help=(
            'Set the default KMS key using the full path to the key, which '
            'has the following form: '
            "``projects/[project-id]/locations/[location]/keyRings/[key-ring]/cryptoKeys/[my-key]''."
        ),
    )
    parser.add_argument(
        '--retention-period',
        help=(
            'Minimum [retention period](https://cloud.google.com'
            '/storage/docs/bucket-lock#retention-periods)'
            ' for objects stored in the bucket, for example'
            " ``--retention-period=P1Y1M1DT5S''. Objects added to the bucket"
            " cannot be deleted until they've been stored for the specified"
            ' length of time. Default is no retention period. Only available'
            ' for Cloud Storage using the JSON API.'
        ),
    )
    flags.add_placement_flag(parser)
    parser.add_argument(
        '--soft-delete-duration',
        type=arg_parsers.Duration(),
        help=(
            'Duration to retain soft-deleted objects. For example, "2w1d" is'
            ' two weeks and one day. See `gcloud topic datetimes` for more'
            ' information on the duration format. Setting `0` will disable'
            ' soft delete policy on the bucket. Default is 7 days.'
        ),
    )
    flags.add_additional_headers_flag(parser)
    flags.add_autoclass_flags(parser)
    flags.add_enable_per_object_retention_flag(parser)
    flags.add_recovery_point_objective_flag(parser)

    parser.add_argument(
        '--enable-hierarchical-namespace',
        action='store_true',
        default=None,
        help=(
            'Enable hierarchical namespace for the bucket. To use this'
            ' flag, you must also use --uniform-bucket-level-access'
        ),
    )

    parser.add_argument('--lifecycle-file', help=_LIFECYCLE_HELP_TEXT)
    flags.add_ip_filter_file_flag(parser)
    flags.add_encryption_enforcement_file_flag(parser)

  def Run(self, args):
    for url_string in args.url:
      url = storage_url.storage_url_from_string(url_string)
      errors_util.raise_error_if_not_bucket(args.command_path, url)
      resource = resource_reference.UnknownResource(url)
      user_request_args = (
          user_request_args_factory.get_user_request_args_from_command_args(
              args, metadata_type=user_request_args_factory.MetadataType.BUCKET
          )
      )
      if (
          user_request_args.resource_args.autoclass_terminal_storage_class
          is not None
          and not user_request_args.resource_args.enable_autoclass
      ):
        raise errors.Error(
            '--autoclass_terminal_storage_class is only allowed if'
            ' --enable-autoclass is set.'
        )
      create_bucket_task.CreateBucketTask(
          resource, user_request_args=user_request_args
      ).execute()



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to delete a Tensorboard time series in Vertex AI."""


from googlecloudsdk.api_lib.ai import operations
from googlecloudsdk.api_lib.ai.tensorboard_time_series import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import operations_util
from googlecloudsdk.command_lib.ai import tensorboards_util


def _Run(args, version):
  tensorboard_time_series_ref = args.CONCEPTS.tensorboard_time_series.Parse()
  region = tensorboard_time_series_ref.AsDict()['locationsId']
  with endpoint_util.AiplatformEndpointOverrides(
      version=version, region=region):
    operation = client.TensorboardTimeSeriesClient(
        version=version).Delete(tensorboard_time_series_ref)
    return operations_util.WaitForOpMaybe(
        operations_client=operations.OperationsClient(),
        op=operation,
        op_ref=tensorboards_util.ParseTensorboardOperation(operation.name))


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class Delete(base.DeleteCommand):
  """Delete an existing Vertex AI Tensorboard time series."""

  detailed_help = {
      'EXAMPLES':
          """\
          To delete a Tensorboard Time Series `123` in Tensorboard `12345`, Tensorboard Experiment `my-tensorboard-experiment`, Tensorboard Run `my-tensorboard-run`, region `us-central1`, and project `my-project`:

              $ {command} projects/my-project/locations/us-central1/tensorboards/12345/experiments/my-tensorboard-experiment/runs/my-tensorboard-run/timeSeries/123

          Or with flags:

              $ {command} 123 --tensorboard-id=12345 --tensorboard-experiment-id=my-tensorboard-experiment --tensorboard-run-id=my-tensorboard-run
          """,
  }

  @staticmethod
  def Args(parser):
    flags.AddTensorboardTimeSeriesResourceArg(parser, 'to delete')

  def Run(self, args):
    return _Run(args, constants.BETA_VERSION)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud app services list` command."""


from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.calliope import base


class List(base.ListCommand):
  """List your existing services.

  This command lists all services that are currently deployed to the App Engine
  server.
  """

  detailed_help = {
      'EXAMPLES': """\
          To list all services in the current project, run:

            $ {command}

          """,
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""
          table(
            id:label=SERVICE:sort=1,
            versions.len():label=NUM_VERSIONS
          )
    """)

  def Run(self, args):
    api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
    services = api_client.ListServices()
    versions = api_client.ListVersions(services)

    result = []
    for service in services:
      versions_for_service = [v for v in versions if v.service == service.id]
      result.append(
          {'id': service.id, 'versions': versions_for_service})
    # Sort so the order is deterministic for testing.
    return sorted(result, key=lambda r: r['id'])



# -*- coding: utf-8 -*- #
# Copyright 2013 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The command to list installed/available gcloud components."""


from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core.updater import snapshots
from googlecloudsdk.core.updater import update_manager


def TransformLastUpdate(r):
  try:
    snapshot = snapshots.ComponentSnapshot.FromURLs(
        r, command_path='components.repositories.list')
    return snapshot.sdk_definition.LastUpdatedString()
  except (AttributeError, TypeError, snapshots.URLFetchError):
    return 'Unknown'


_COMPONENTS_REPOSITORIES_TRANSFORMS = {
    'last_update': TransformLastUpdate,
}


class List(base.ListCommand):
  """List any Trusted Tester component repositories you have registered.
  """
  detailed_help = {
      'DESCRIPTION': """
          List all Trusted Tester component repositories that are registered
          with the component manager.  If you have additional repositories, the
          component manager will look at them to discover additional components
          to install, or different versions of existing components that are
          available.
      """,
      'EXAMPLES': """
          To list all Trusted Tester component repositories that are registered
          with the component manager, run:

            $ {command}
      """
  }

  @staticmethod
  def Args(parser):
    """Adds/removes args for this command."""
    base.PAGE_SIZE_FLAG.RemoveFromParser(parser)
    base.URI_FLAG.RemoveFromParser(parser)
    parser.display_info.AddFormat("""
          table(
            .:label=REPOSITORY,
            last_update():label=LAST_UPDATE
          )
    """)
    parser.display_info.AddTransforms(_COMPONENTS_REPOSITORIES_TRANSFORMS)

  def Run(self, args):
    """Runs the list command."""
    repos = update_manager.UpdateManager.GetAdditionalRepositories()
    return repos if repos else []

  def Epilog(self, resources_were_displayed):
    if not resources_were_displayed:
      log.status.write(
          'You have no registered component repositories.  To add one, run:\n'
          '  $ gcloud components repositories add URL\n\n')



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Interconnect Group."""




class InterconnectGroup(object):
  """Abstracts Interconnect Group resource."""

  def __init__(self, ref, project, compute_client=None, resources=None):
    self.ref = ref
    self.project = project
    self._compute_client = compute_client
    self._resources = resources

  @property
  def _client(self):
    return self._compute_client.apitools_client

  @property
  def _messages(self):
    return self._compute_client.messages

  def MakeInterconnectGroupsCreateMembersInterconnectInput(
      self,
      facility: str = None,
      description: str = None,
      name: str = None,
      link_type: str = None,
      requested_link_count: int = 1,
      interconnect_type: str = None,
      admin_enabled: bool = True,
      noc_contact_email: str = None,
      customer_name: str = None,
      remote_location: str = None,
      requested_features: str = None,
  ):
    """Make an InterconnectGroupsCreateMembersInterconnectInput."""
    return self._messages.InterconnectGroupsCreateMembersInterconnectInput(
        facility=facility,
        description=description,
        name=name,
        linkType=link_type,
        requestedLinkCount=requested_link_count,
        interconnectType=interconnect_type,
        adminEnabled=admin_enabled,
        nocContactEmail=noc_contact_email,
        customerName=customer_name,
        remoteLocation=remote_location,
        requestedFeatures=requested_features,
    )

  def _MakeAdditionalProperties(self, interconnects):
    return [
        self._messages.InterconnectGroup.InterconnectsValue.AdditionalProperty(
            # The keys are arbitrary strings that only need to
            # be unique, and we use the Interconnect name.
            key=interconnect,
            value=self._messages.InterconnectGroupInterconnect(
                interconnect=self._resources.Create(
                    'compute.interconnects',
                    interconnect=interconnect,
                    project=self.ref.project,
                ).SelfLink()
            ),
        )
        for interconnect in interconnects
    ]

  def _MakeCreateRequestTuple(
      self,
      description,
      topology_capability,
      interconnects,
  ):
    """Make a tuple for interconnect group insert request.

    Args:
      description: String that represents the description of the Cloud
        Interconnect Group resource.
      topology_capability: String that represents the topology capability of the
        Cloud Interconnect Group resource.
      interconnects: List of strings that represent the names of the Cloud
        Interconnect resources that are members of the Cloud Interconnect Group
        resource.

    Returns:
    Insert interconnect group tuple that can be used in a request.
    """
    messages = self._messages
    return (
        self._client.interconnectGroups,
        'Insert',
        messages.ComputeInterconnectGroupsInsertRequest(
            project=self.project,
            interconnectGroup=messages.InterconnectGroup(
                intent=messages.InterconnectGroupIntent(
                    topologyCapability=topology_capability
                ),
                name=self.ref.Name(),
                description=description,
                interconnects=messages.InterconnectGroup.InterconnectsValue(
                    additionalProperties=self._MakeAdditionalProperties(
                        interconnects
                    )
                ),
            ),
        ),
    )

  def _MakePatchRequestTuple(
      self, topology_capability, interconnects, update_mask, **kwargs
  ):
    """Make a tuple for interconnect group patch request."""
    messages = self._messages
    group_params = {
        'interconnects': messages.InterconnectGroup.InterconnectsValue(
            additionalProperties=self._MakeAdditionalProperties(interconnects)
        ),
    }
    group_params.update(kwargs)
    if topology_capability is not None:
      group_params['intent'] = messages.InterconnectGroupIntent(
          topologyCapability=topology_capability
      )
    return (
        self._client.interconnectGroups,
        'Patch',
        messages.ComputeInterconnectGroupsPatchRequest(
            project=self.project,
            interconnectGroup=self.ref.Name(),
            interconnectGroupResource=messages.InterconnectGroup(
                **group_params
            ),
            updateMask=update_mask,
        ),
    )

  def _MakeDeleteRequestTuple(self):
    return (
        self._client.interconnectGroups,
        'Delete',
        self._messages.ComputeInterconnectGroupsDeleteRequest(
            project=self.ref.project, interconnectGroup=self.ref.Name()
        ),
    )

  def _MakeDescribeRequestTuple(self):
    return (
        self._client.interconnectGroups,
        'Get',
        self._messages.ComputeInterconnectGroupsGetRequest(
            project=self.ref.project, interconnectGroup=self.ref.Name()
        ),
    )

  def _MakeGetOperationalStatusRequestTuple(self):
    return (
        self._client.interconnectGroups,
        'GetOperationalStatus',
        self._messages.ComputeInterconnectGroupsGetOperationalStatusRequest(
            project=self.ref.project, interconnectGroup=self.ref.Name()
        ),
    )

  def _MakeCreateMembersRequestTuple(
      self,
      intent_mismatch_behavior,
      template_interconnect,
      member_interconnects,
  ):
    messages = self._messages
    return (
        self._client.interconnectGroups,
        'CreateMembers',
        messages.ComputeInterconnectGroupsCreateMembersRequest(
            project=self.ref.project,
            interconnectGroupsCreateMembersRequest=messages.InterconnectGroupsCreateMembersRequest(
                request=messages.InterconnectGroupsCreateMembers(
                    intentMismatchBehavior=intent_mismatch_behavior,
                    templateInterconnect=template_interconnect,
                    interconnects=member_interconnects,
                ),
            ),
            interconnectGroup=self.ref.Name(),
        ),
    )

  def Create(
      self,
      description=None,
      topology_capability=None,
      interconnects=(),
      only_generate_request=False,
  ):
    """Create an interconnect group."""
    requests = [
        self._MakeCreateRequestTuple(
            description,
            topology_capability,
            interconnects,
        )
    ]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests

  def Delete(self, only_generate_request=False):
    requests = [self._MakeDeleteRequestTuple()]
    if not only_generate_request:
      return self._compute_client.MakeRequests(requests)
    return requests

  def Describe(self, only_generate_request=False):
    requests = [self._MakeDescribeRequestTuple()]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests

  def Patch(
      self,
      topology_capability=None,
      interconnects=(),
      update_mask='',
      only_generate_request=False,
      **kwargs
  ):
    """Patch description, topology capability and member interconnects of an interconnect group."""
    requests = [
        self._MakePatchRequestTuple(
            topology_capability, interconnects, update_mask, **kwargs
        )
    ]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests

  def GetOperationalStatus(self, only_generate_request=False):
    requests = [self._MakeGetOperationalStatusRequestTuple()]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests

  def CreateMembers(
      self,
      intent_mismatch_behavior=None,
      template_interconnect=None,
      member_interconnects=(),
  ):
    """Create member interconnects in an interconnect group."""
    requests = [
        self._MakeCreateMembersRequestTuple(
            intent_mismatch_behavior,
            template_interconnect,
            member_interconnects,
        )
    ]
    resources = self._compute_client.MakeRequests(requests)
    return resources[0]
